@@ -415,7 +415,7 @@ print_seq(FILE *f,
                  i < h->start + h->length + sa->incl_s)
                notes[i] |= INTL_OLIGO;
         } else if (pa->primer_task == pick_sequencing_primers) {
-                if (pa->pick_right_primer && &retval->rev != NULL 
+                if (pa->pick_right_primer && (&retval->rev) != NULL
                          && retval->rev.num_elem > 0){
                   h2 = retval->rev.oligo;
               for (j = 0; j < pa->num_return; j++) {
@@ -426,7 +426,7 @@ print_seq(FILE *f,
                    notes[i] |= RIGHT_OLIGO;
                   }
             }           
-                if (pa->pick_left_primer && &retval->fwd != NULL 
+                if (pa->pick_left_primer && (&retval->fwd) != NULL
                          && retval->fwd.num_elem > 0){
                   h2 = retval->fwd.oligo;
               for (j = 0; j < pa->num_return; j++) {
@@ -848,21 +848,21 @@ format_oligos(FILE *f,
       print_oligo_header(f, "OLIGO", print_lib_sim, pa->thermodynamic_oligo_alignment);
     }
     /* Print out the first line with the best primers */
-    if ((pa->pick_left_primer) && (&retval->fwd != NULL )
+    if ((pa->pick_left_primer) && ((&retval->fwd) != NULL )
                && (retval->fwd.num_elem > 0)){
       print_oligo(f, "LEFT_PRIMER", sa, retval->fwd.oligo, FORWARD, 
                   pa, pa->p_args.repeat_lib, print_lib_sim);
       h = retval->fwd.oligo;
       rest_count = 1;
     }
-    if ((pa->pick_internal_oligo) && (&retval->intl != NULL )
+    if ((pa->pick_internal_oligo) && ((&retval->intl) != NULL )
              && (retval->intl.num_elem > 0)){
       print_oligo(f, "INTERNAL_OLIGO", sa, retval->intl.oligo, FORWARD, 
                   pa, pa->p_args.repeat_lib, print_lib_sim);
       h = retval->intl.oligo;
       rest_count = 1;
     }
-    if ((pa->pick_right_primer) && (&retval->rev != NULL )
+    if ((pa->pick_right_primer) && ((&retval->rev) != NULL )
              && (retval->rev.num_elem > 0)) {
       print_oligo(f, "RIGHT_PRIMER", sa, retval->rev.oligo, REVERSE, 
                   pa, pa->p_args.repeat_lib, print_lib_sim);
@@ -885,7 +885,7 @@ format_oligos(FILE *f,
   }
   fprintf(f, "\n");
   /* Print out the other primers */
-  if ((pa->pick_left_primer) && (&retval->fwd != NULL )
+  if ((pa->pick_left_primer) && ((&retval->fwd) != NULL )
              && (retval->fwd.num_elem > rest_count)){
     int n = retval->fwd.num_elem;
     h = retval->fwd.oligo;
@@ -904,7 +904,7 @@ format_oligos(FILE *f,
       fprintf(f, "\n ");
     }
   }
-  if ((pa->pick_internal_oligo) && (&retval->intl != NULL )
+  if ((pa->pick_internal_oligo) && ((&retval->intl) != NULL )
            && (retval->intl.num_elem > rest_count)){
     int n = retval->intl.num_elem;
     h = retval->intl.oligo;  
@@ -923,7 +923,7 @@ format_oligos(FILE *f,
       fprintf(f, "\n ");
     }
   }
-  if ((pa->pick_right_primer) && (&retval->rev != NULL )
+  if ((pa->pick_right_primer) && ((&retval->rev) != NULL )
            && (retval->rev.num_elem > rest_count)) {
     int n = retval->rev.num_elem;
     h = retval->rev.oligo; 
