@@ -68,7 +68,10 @@ main(int argc, const char**argv)
   double salt_conc = 50;
   double divalent_conc = 0;
   double dntp_conc = 0;
-  double tm;
+  double dmso_conc = 0.0;
+  double dmso_fact = 0.6;
+  double formamide_conc = 0.0;
+  tm_ret tm_calc;  /* structure with Tm and bound (primer fraction) */
   int start, len;
   char *endptr;
 
@@ -82,7 +85,8 @@ main(int argc, const char**argv)
   len = strtol(argv[3], &endptr, 10);
   printf("s=%s, start=%d, length=%d\n", s, start, len);
   
-  tm = long_seq_tm(s, start, len, salt_conc, divalent_conc, dntp_conc);
-  printf("tm = %f\n", tm);
+  tm_calc = long_seq_tm(s, start, len, salt_conc, divalent_conc, dntp_conc,
+                        dmso_conc, dmso_fact, formamide_conc);
+  printf("tm = %f\n", tm_calc.Tm);
   return 0;
 }
